import re,sys

newline=re.compile(r"[\r\n]+")

class queue:
  def __init__(self):
    self.event_map={}
    self.event_list=[[] for i in range(3)]
  def enqueue(self,event):
    if event.constraint in self.event_map:
      return
    self.event_map[event.constraint]=None
    self.event_list[event.priority].append(event.constraint)
    # Would have been better if things could change priority once in the queue.
  def dequeue(self):
    for i in range(3):
      if len(self.event_list[i])>0:
        rc=self.event_list[i][0]
        self.event_list[i]=self.event_list[i][1:]
        del self.event_map[rc]
        return rc
    return None

class msg:
  def __init__(self,priority,constraint):
    self.priority=priority # priority of message: 0=highest
    self.constraint=constraint # constraint that needs re-checking

class loctype:
  def __init__(self):
    self.constraints=[]

def build_tree(i,transpose,reverse_matching):
  cur_set=[i]
  tree_pos={}
  counter=0
  while True:
    counter+=1
    next_set=[]
    for cur in cur_set:
      for pos in transpose[cur]:
        if pos not in tree_pos:
          tree_pos[pos]=cur
          if pos not in reverse_matching:
            return (pos,tree_pos)
          else:
            next_set.append(reverse_matching[pos])
    cur_set=next_set
    if counter==len(transpose)+1:
      raise "Inconsistent board"

def perfect_matching(transpose):
  matching=[None for i in range(len(transpose))]
  pos_set={}
  for i in range(len(transpose)):
    for position in transpose[i]:
      pos_set[position]=None
  reverse_matching={}
  for i in range(len(transpose)):
    if matching[i]==None:
      (pos,tree_pos)=build_tree(i,transpose,reverse_matching)
      quit=False
      while not quit:
        quit=(tree_pos[pos]==i)
        if not quit:
          new_pos=matching[tree_pos[pos]]
        reverse_matching[pos]=tree_pos[pos]
        matching[tree_pos[pos]]=pos
        if not quit:
          pos=new_pos
  return matching

def bellman_ford(transpose,weights,matching):
  graph={}
  for i in range(len(transpose)):
    graph[("i",i)]=(None,0)
    for position in transpose[i]:
      graph[("pos",position)]=(None,0)
  for counter in range(len(transpose)*2):
    for i in range(len(transpose)):
      for position in transpose[i]:
        w=weights[i][position]
        if matching[i]==position:
          if graph[("pos",position)][1]>graph[("i",i)][1]+w:
            graph[("pos",position)]=(("i",i),graph[("i",i)][1]+w)
        else:
          if graph[("i",i)][1]>graph[("pos",position)][1]-w:
            graph[("i",i)]=(("pos",position),graph[("pos",position)][1]-w)
  improvement={}
  for i in range(len(transpose)):
    for position in transpose[i]:
      w=weights[i][position]
      if matching[i]==position:
        if graph[("pos",position)][1]>graph[("i",i)][1]+w:
          improvement[("pos",position)]=None
      else:
        if graph[("i",i)][1]>graph[("pos",position)][1]-w:
          improvement[("i",i)]=None
  return (improvement,graph)

def find_cycles(improvement,graph):
  # report only new edges
  edges=[]
  labels={}
  for node in improvement:
    p=node
    while p not in labels:
      labels[p]=1
      p=graph[p][0]
    if labels[p]==1:
      if p[0]=="pos":
        p=graph[p][0]
      while labels[p]==1:
        labels[p]=2
        labels[graph[p][0]]=2
        edges.append((p[1],graph[p][0][1]))
        p=graph[graph[p][0]][0]
    p=node
    while labels[p]==1:
      labels[p]=2
      p=graph[p][0]
  return edges

def clean(transpose):
  clean_transpose=[{} for i in range(len(transpose))]
  weights=[{} for i in range(len(transpose))]
  for i in range(len(transpose)):
    for position in transpose[i]:
      weights[i][position]=1
  matching=perfect_matching(transpose)
  for i in range(len(transpose)):
    clean_transpose[i][matching[i]]=None
    weights[i][matching[i]]=0
  improvement=True
  while improvement:
    (improvement,graph)=bellman_ford(transpose,weights,matching)
    if improvement:
      edges=find_cycles(improvement,graph) # report only new edges
      for (i,position) in edges:
        if weights[i][position]==1:
          weights[i][position]=0
          clean_transpose[i][position]=None
  return clean_transpose

class sudoku:
  def __init__(self,n=3):
    self.n=n
    self.numbers=self.n*self.n
    self.rows=self.n*self.n
    self.columns=self.n*self.n
    self.area=self.rows*self.columns
    self.sectors=self.n*self.n
    self.constraints=[]
    for j in range(self.n):
      for i in range(self.n):
        self.constraints.append([self.pos(self.n*i+x,self.n*j+y) \
          for x in range(self.n) for y in range(self.n)])
    # The "localize" method requires sectors to be defined first.
    for i in range(self.columns):
      self.constraints.append([self.pos(i,x) for x in range(self.rows)])
    for i in range(self.rows):
      self.constraints.append([self.pos(x,i) for x in range(self.columns)])
    self.constraints_mask=[0 for i in range(len(self.constraints))]
    for i in range(len(self.constraints)):
      for j in self.constraints[i]:
        self.constraints_mask[i]+=2**j
    if self.n==3:
      self.dictionary={"1":0,"2":1,"3":2,"4":3,"5":4,"6":5,"7":6,"8":7,"9":8}
      self.inv_dict=["1","2","3","4","5","6","7","8","9"]
    else:
      self.dictionary={}
      self.inv_dict=[]
  def pos(self,x,y):
    return y*self.columns+x
  def find_constraints(self,position):
    return [i for i in range(len(self.constraints)) \
            if position in self.constraints[i]]
  def clear(self):
    self.board=[[1]*self.numbers for i in range(self.area)]
    self.knowledge=[[0]*self.sectors for j in range(self.numbers)]
    self.pending=queue()
    if self.n!=3:
      self.dictionary={}
      self.inv_dict=[" "]*self.numbers
    self.remaining_atoms=self.area
  def solve(self,board,verbose=False):
    self.clear()
    self.read(board)
    self.main(verbose)
    if not verbose:
      self.output()
  def read(self,board):
    distinct=0
    board=newline.sub("\n",board)
    for i in range(len(board)):
      x=i%(self.columns+1) # because of new-lines
      y=i/(self.columns+1)
      c=board[i]
      if x<self.columns and c!=' ':
        position=self.pos(x,y)
        if c not in self.dictionary:
          if distinct==self.numbers:
            print "Here are some of the elements appearing on the board:"
            print self.dictionary.keys()
            raise "Board contains too many distinct elements."
          val=distinct
          self.dictionary[c]=val
          distinct+=1
        else:
          val=self.dictionary[c]
        for i in range(self.numbers):
          if i!=val:
            self.board[position][i]=0
        for c in self.find_constraints(position):
          self.pending.enqueue(msg(0,c))
    self.inv_dict=range(self.numbers)
    for i in self.dictionary:
      self.inv_dict[self.dictionary[i]]=i
  def localize(self,poslist):
    rc=loctype()
    mask=0
    for i in poslist:
      mask+=2**i
    for i in range(len(self.constraints_mask)):
      if mask | self.constraints_mask[i]==self.constraints_mask[i]:
        rc.constraints.append(i)
    rc.exactness=len(rc.constraints)
    rc.sector=rc.constraints[0] # this is why sectors are defined first
    total={}
    for i in rc.constraints:
      for j in self.constraints[i]:
        total[j]=None
    for i in poslist:
      del total[i]
    rc.broadcast=total.keys()
    return rc
  def main(self,verbose=False):
    while self.remaining_atoms>0:
      constraint=self.pending.dequeue()
      if constraint==None:
        return
      transpose=[{} for i in range(self.numbers)]
      for i in range(self.numbers):
        for j in self.constraints[constraint]:
          if self.board[j][i]==1:
            transpose[i][j]=None
      for i in range(self.numbers):
        loc=self.localize(transpose[i])
        if loc.exactness>1 and self.knowledge[i][loc.sector]<loc.exactness:
          self.knowledge[i][loc.sector]=loc.exactness
          if loc.exactness==3:
            self.remaining_atoms-=1
          for position in loc.broadcast:
            if self.board[position][i]==1:
              self.board[position][i]=0
              if sum(self.board[position])==1:
                if verbose:
                  self.output()
              for c in self.find_constraints(position):
                self.pending.enqueue(msg(1,c))
      clean_transpose=clean(transpose)
      constraints={}
      for i in range(self.numbers):
        for position in transpose[i]:
          if position not in clean_transpose[i]:
            self.board[position][i]=0
            if verbose and sum(self.board[position])==1:
              self.output()
            for c in self.find_constraints(position):
              constraints[c]=None
      for c in constraints:
        self.pending.enqueue(msg(2,c))
  def output(self):
    rc=""
    for y in range(self.rows):
      for x in range(self.columns):
        data=self.board[self.pos(x,y)]
        if sum(data)==1:
          rc+=self.inv_dict[data.index(1)]
        else:
          rc+=' '
      rc+='\n'
    print rc

if __name__=="__main__":
  p=sudoku()
  p.solve(sys.stdin.read())

